!   该版本是对极化率矩阵进行求和,经过验证,结果与求极化率矩阵的最大本征值结果是相同的
module param
    implicit none
    integer,parameter::dp = kind(1.0)
    integer numk_bz,num_wann,nrpts,numk_FS,kn
    real(dp) mu
    parameter(kn = 500,mu = 0.0)
    real(dp),parameter::pi = 3.1415926535897
    real(dp),parameter::deltaE = 0.01  ! 费米面展宽
    complex(dp),parameter::im = (0.,1.) !Imagine unit
    real(dp),allocatable::ones_ham(:,:)  ! 为了后续考虑化学势
    complex(dp),allocatable::HmnR(:,:,:)  ! hr中的hopping
    integer,allocatable::irvec(:,:)  ! hr中的位置矢量
    integer,allocatable::indorbit(:,:,:,:)
    real(dp),allocatable::BZklist(:,:)
    real(dp),allocatable::FS_points(:,:)
    integer FS_index(-kn:kn - 1,-kn:kn - 1)
    !------------------------------------------
    integer,allocatable::FSindex(:,:)  ! 费米点索引指标
end module
!==============================================================================================================================================================
program main
    use param
    use mpi 
    implicit none
    integer numcore,indcore,ierr
    !-------------------------------------------------
    real(dp) code_time_start,code_time_end,code_time
    integer nki,nkf
    character(len = 20)::filename,char1,char2,char3,char4
    !-------------------------------------------------------------------
    call MPI_INIT(ierr)     ! 初始化进程
    call MPI_COMM_RANK(MPI_COMM_WORLD, indcore, ierr) ! 得到本进程在通信空间中的rank值,即在组中的逻辑编号(该indcore为0到numcore-1间的整数,相当于进程的ID。)
    call MPI_COMM_SIZE(MPI_COMM_WORLD, numcore, ierr) !获得进程数量,用numcoer保存
    ! 循环分拆
    nki = floor(indcore * (2.0 * kn)/numcore) - kn
    nkf = floor((indcore + 1) * (2.0 * kn)/numcore) - kn - 1 
    if(indcore.eq.0)then
        call CPU_TIME(code_time_start)
        write(*,*)"Start calcation: ",code_time_start
        write(*,*)"Number of CPU = ",numcore
        write(*,*)"Number of nk = ",2 * kn
    end if
    ! 这些计算也不耗时,就直接在每个核心上都计算了,实际上可以在单一进程读取数据然后做广播
    
    

    


    ! 输出程序的基本参数信息
    if(indcore.eq.0)then
        write(*,*)"Number of Fermi points = ",numk_FS
        write(*,*)"Number of BZ points = ",numk_bz

        call squareBZ()
        call read_hr()  
        call fermisurface()  ! 确定体系的费米面,其中会给出所有费米点的位置

        call CPU_TIME(code_time_end)
        write(*,*)"Calcation finished",code_time_end
        write(*,*)"Code execution time:",abs(code_time_end - code_time_start),"second"
    end if

    call MPI_Finalize(ierr)
    
    stop
end program main
!================================================================================================================================================================================================
subroutine fermisurface()
    use param
    integer ibz,ie,ikx,iky
    real(dp) kx,ky,mateigval(num_wann)
    complex(dp) Ham_temp(num_wann,num_wann),matvec(num_wann,num_wann)
    ! call honeycomyBZ()
    open(13,file = "FS.dat")
    ! numk_FS = 0   ! 全局变量,共计费米点的数量
    do ibz = 1,numk_bz
        kx = BZklist(1,ibz)
        ky = BZklist(2,ibz)
        call HamR_to_K(kx,ky,0.0,Ham_temp)
        call diagonalize_Hermitian_matrix(num_wann,Ham_temp,matvec,mateigval)
        do ie = 1,num_wann
            if (abs(mateigval(ie)) < deltaE)then
                write(13,"(3F12.6)")kx,ky,1.0
                numk_FS = numk_FS + 1  ! 统计费米点的数量
            end if
        end do
    end do
    close(13)
    !--------------------------------------------------------------
    !确定费米面点的数量后就可以确定相互作用矩阵的维度
    if(numk_FS.eq.0)then
        write(*,*)"The Number of Fermi surface points = ",numk_FS
        write(*,*)"----------------------------------------------"
        write(*,*)"                    Warnning                  "
        write(*,*)"----------------------------------------------"
        stop
    else
        allocate(FS_points(2,numk_FS))
        FS_points = 0.0
    end if
    !-----------------------------------------------
    !  根据动量坐标记录这是第几个费米点
    numk_FS = 0   
    do iky = -kn,kn - 1
        ky = 2.0 * pi * iky/kn
        do ikx = -kn,kn - 1
            kx = 2.0 * pi * ikx/kn
            call HamR_to_K(kx,ky,0.0,Ham_temp)
            call diagonalize_Hermitian_matrix(num_wann,Ham_temp,matvec,mateigval)
            do ie = 1,num_wann
                if (abs(mateigval(ie)) < deltaE)then
                    numk_FS = numk_FS + 1 
                    FS_index(ikx,iky) = numk_FS   
                    FS_points(1,numk_FS) = kx
                    FS_points(2,numk_FS) = ky
                end if
            end do
        end do
    end do
    return
end subroutine

!================================================================================================================================================================================================
subroutine squareBZ()
    use param
    integer ikx,iky,i0
    ! 对于四方点阵,BZ的点数可以直接确定
    numk_bz = (2 * kn)**2   
    allocate(BZklist(2,numk_bz))
    i0 = 0
    do ikx = -kn,kn - 1
        do iky = -kn,kn - 1
            i0 = i0 + 1
            BZklist(1,i0) = 2.0 * pi * ikx/kn
            BZklist(2,i0) = 2.0 * pi * iky/kn 
        end do
    end do
    return  
end subroutine
!==============================================================================================================================================================
subroutine HamR_to_K(kx,ky,kz,Ham_temp)
    ! 将hr变成动量空间
    use param
    implicit none
    real(dp) kx,ky,kz
    integer i1,i2,i3,w1,w2,r1,r2,r3
    complex(dp) Ham_temp(num_wann,num_wann)

    ones_ham = 0.0   ! 对于具有allocatable属性的变量,一定要先赋值为零
    ! 设置单位矩阵
    do i1 = 1,num_wann
        ones_ham(i1,i1) = 1.0
    end do 


    Ham_temp = 0.0
    do i1 = 1,nrpts
        do i2 = 1,num_wann
            do i3 = 1,num_wann
                r1 = irvec(1,i1)
                r2 = irvec(2,i1)
                r3 = irvec(3,i1)
                w1 = indorbit(1,i1,i2,i3)
                w2 = indorbit(2,i1,i2,i3)
                Ham_temp(w1,w2) = Ham_temp(w1,w2) +  (cos(kx * r1 + ky * r2 + kz * r3) + im * sin(kx * r1 + ky * r2 + kz * r3)) * HmnR(w1,w2,i1) 
            end do
        end do
    end do
    Ham_temp = Ham_temp - mu * ones_ham   ! 在Wannier90_hr的基础上考虑体系的化学势
end subroutine
!==============================================================================================================================================================
subroutine read_hr()
    ! 读取DFT的hr数据,并返回Hr以及
    use param
    implicit none
    integer :: i,j,k
    integer :: r1,r2,r3,w1,w2
    real(dp) :: hr,hi
    real(dp),allocatable::ndegen(:)
    logical stat

    inquire(file = 'wannier90_hr.dat', exist = stat)  
    if(stat)then
        open(12,file = 'wannier90_hr.dat',action = 'read')
        read(12,*)
        read(12,*) num_wann  ! 轨道数量,也决定了动量空间哈密顿量维度,这是一个全局变量
        ! read(12,*)r1
        read(12,*) nrpts
        
        allocate(ndegen(nrpts))
        !  全局变量
        allocate(HmnR(num_wann,num_wann,nrpts),irvec(3,nrpts))
        allocate(indorbit(2,nrpts,num_wann,num_wann))
        read(12,*) (ndegen(i),i = 1,nrpts)
        
        do i = 1,nrpts
            do j = 1,num_wann
                do k = 1,num_wann
                    read(12,*) r1,r2,r3,w1,w2,hr,hi
                    irvec(1,i) = r1
                    irvec(2,i) = r2
                    irvec(3,i) = r3
                    indorbit(1,i,j,k) = w1   ! 哈密顿量中的轨道索引,后续构建动量空间哈密顿量使用
                    indorbit(2,i,j,k) = w2
                    HmnR(w1,w2,i) =  hr + im * hi 
                enddo
            end do
        end do
        close(12)
        allocate(ones_ham(num_wann,num_wann))
    else
        write(*,*)"**************** Error *********************"
        write(*,*)"Tight-binding dat is not exist"
        write(*,*)"**************** Error *********************"
    end if
    return
end subroutine read_hr
!==============================================================================================================================================================
function delta(x)
    !>  Lorentz or gaussian expansion of the Delta function
    use param, only : dp, pi
    implicit none
    ! real(dp), intent(in) :: eta
    real(dp), intent(in) :: x
    real(dp) :: delta, y
    real(dp) :: eta = 0.001

    !> Lorentz expansion
    !delta= 1d0/pi*eta/(eta*eta+x*x)
 
    y= x*x/eta/eta/2d0
 
    !> Gaussian broadening
    !> exp(-60) = 8.75651076269652e-27
    if (y>60d0) then
       delta = 0d0
    else
       delta= exp(-y)/sqrt(2d0*pi)/eta
    endif
 
    return
 end function delta
!==============================================================================================================================================================
subroutine diagonalize_complex_matrix(matdim,matin,matout,mateigval)
    ! 对角化一般复数矩阵,这里的本征值是个复数
    ! matin 输入矩阵   matout 本征矢量    mateigval  本征值
    implicit none
    integer,parameter::dp = kind(1.0)
    integer matdim,LDA,LDVL,LDVR,LWMAX,INFO,LWORK
    complex(dp),intent(in)::matin(matdim,matdim)
    complex(dp),intent(out)::matout(matdim,matdim)
    complex(dp),intent(out)::mateigval(matdim)
    real(dp),allocatable::RWORK(:)
    complex(dp),allocatable::WORK(:)
    complex(dp),allocatable::VL(:,:)
    complex(dp),allocatable::VR(:,:)
    LDA = matdim
    LDVL = matdim
    LDVR = matdim
    LWMAX = 2 * matdim + matdim**2
    ! write(*,*)matin
    allocate(RWORK(2 * matdim))
    allocate(VL(LDVL,matdim))
    allocate(VR(LDVR, matdim))
    allocate(WORK(LWMAX))
    matout = matin

    LWORK = -1
    call cgeev( 'V', 'N', matdim, matout, LDA, mateigval, VL, LDVL,VR, LDVR, WORK, LWORK, RWORK, INFO)
    LWORK = MIN( LWMAX, INT( WORK( 1 ) ) )
    call cgeev( 'V', 'N', matdim, matout, LDA, mateigval, VL, LDVL,VR, LDVR, WORK, LWORK, RWORK, INFO )
    IF( INFO.GT.0 ) THEN
       WRITE(*,*)'The algorithm failed to compute eigenvalues.'
    !    STOP
    END IF
end subroutine diagonalize_complex_matrix
!================================================================================================================================================================================================
subroutine Matrix_Inv(matdim,matin,matout)
    ! 矩阵求逆 
    implicit none
    integer,parameter::dp = kind(1.0)
    integer matdim,info
    complex(dp),intent(in) :: matin(matdim,matdim)
    complex(dp):: matout(size(matin,1),size(matin,2))
    real(dp):: work2(size(matin,1))            ! work2 array for LAPACK
    integer::ipiv(size(matin,1))     ! pivot indices
    ! Store matin in matout to prevent it from being overwritten by LAPACK
    matout = matin
    ! SGETRF computes an LU factorization of a general M - by - N matrix A
    ! using partial pivoting with row interchanges .
    call CGETRF(matdim,matdim,matout,matdim,ipiv,info)
    ! if (info.ne.0) stop 'Matrix is numerically singular!'
    if (info.ne.0)  write(*,*)'Matrix is numerically singular!'
    ! SGETRI computes the inverse of a matrix using the LU factorization
    ! computed by SGETRF.
    call CGETRI(matdim,matout,matdim,ipiv,work2,matdim,info)
    ! if (info.ne.0) stop 'Matrix inversion failed!'
    if (info.ne.0) write(*,*)'Matrix inversion failed!'
    return
end subroutine Matrix_Inv
!================================================================================================================================================================================================
subroutine diagonalize_Hermitian_matrix(matdim,matin,matout,mateigval)
    !  厄米矩阵对角化
    ! matin 输入矩阵   matout 本征矢量    mateigval  本征值
    implicit none
    integer,parameter::dp = kind(1.0)
    integer matdim
    integer lda0,lwmax0,lwork,lrwork,liwork,info
    complex(dp) matin(matdim,matdim),matout(matdim,matdim)
    real(dp) mateigval(matdim)
    complex(dp),allocatable::work(:)
    real(dp),allocatable::rwork(:)
    integer,allocatable::iwork(:)
    !-----------------
    lda0 = matdim
    lwmax0 = 2 * matdim + matdim**2
    allocate(work(lwmax0))
    allocate(rwork(1 + 5 * matdim + 2 * matdim**2))
    allocate(iwork(3 + 5 * matdim))
    matout = matin
    lwork = -1
    liwork = -1
    lrwork = -1
    call cheevd('V','U',matdim,matout,lda0,mateigval,work,lwork ,rwork,lrwork,iwork,liwork,info)
    lwork = min(2 * matdim + matdim**2, int( work( 1 ) ) )
    lrwork = min(1 + 5 * matdim + 2 * matdim**2, int( rwork( 1 ) ) )
    liwork = min(3 + 5 * matdim, iwork( 1 ) )
    call cheevd('V','U',matdim,matout,lda0,mateigval,work,lwork,rwork,lrwork,iwork,liwork,info)
    if( info .GT. 0 ) then
        open(11,file = "mes.dat",status = "unknown")
        write(11,*)'The algorithm failed to compute eigenvalues.'
        close(11)
    end if
    return
end subroutine diagonalize_Hermitian_matrix
